from mq_consumer.connectors import Connector


class MQChecker:
    def __init__(self, connector: Connector, passive=True, durable=False):
        self.connector = connector
        self.passive = passive
        self.durable = durable

    def get_message_count(self):
        self.connector.create_connection(declare=False)
        try:
            msg_count = self.connector.channel.queue_declare(
                queue=self.connector.queue, passive=self.passive, durable=self.durable
            ).method.message_count
        finally:
            self.connector.close()
        return msg_count

    def get_consumer_count(self):
        self.connector.create_connection(declare=False)
        try:
            consumers_count = self.connector.channel.queue_declare(queue=self.connector.queue, passive=True).method.consumer_count
        finally:
            self.connector.close()
        return consumers_count



from mq_consumer.checkers import MQChecker
from mq_consumer.connectors import Connector

from tests.test_connection_params import connection_parameters


class TestChecker(MQChecker):
    def __init__(self):
        connector = Connector(
            connection_parameters,
            'exchange',
            'test',
        )
        super().__init__(connector)


def run():
    print(TestChecker().get_message_count())
    print(TestChecker().get_consumer_count())


if __name__ == '__main__':
    run()



import pika

host = '127.0.0.1'
port = 5672
user = 'guest'
password = 'guest'
connection_attempts = 1

connection_parameters = pika.ConnectionParameters(
        host=host,
        port=port,
        credentials=pika.credentials.PlainCredentials(user, password),
        heartbeat_interval=0,
        connection_attempts=connection_attempts
)


import datetime

from tests.test_consumer import TestConsumer


def run():
    with TestConsumer().get_json_publisher() as test_publisher:
        for i in range(100):
            test_publisher.send_message(dict(
                text=f'Сообщение {i}',
                timestamp=datetime.datetime.now(),
            ))


if __name__ == '__main__':
    run()



from setuptools import setup, find_packages

import mq_consumer

setup(
    name='mq_consumer',
    version=mq_consumer.__version__,
    author='Ruzzy Rullezz',
    author_email='ruslan@lemimi.ru',
    packages=find_packages(),
    package_dir={'mq_consumer': 'mq_consumer'},
    install_requires=[
        'pika',
    ],
)


import signal
import functools
import os
import logging

from typing import Callable

import pika.exceptions

from .connectors import Connector, Reconnector
from .message import MQMessage
from .publishers import Publisher, JSONPublisher


class Consumer:
    def __init__(self, connector: Connector, handler: Callable):
        self.connector = connector
        self.handler = handler

    def start_consuming(self):
        self.connector.create_connection()
        self.connector.channel.basic_consume(self.handler, queue=self.connector.queue)
        self.connector.channel.start_consuming()

    def stop_consuming(self):
        self.connector.channel.stop_consuming()

    def run(self):
        try:
            self.start_consuming()
        except KeyboardInterrupt:
            pass
        finally:
            self.stop_consuming()

    def get_publisher(self, publisher_cls) -> Publisher:
        return publisher_cls(self.connector)

    # noinspection PyTypeChecker
    def get_json_publisher(self) -> JSONPublisher:
        return JSONPublisher(self.connector)


class ReconConsumer(Consumer):
    def __init__(self, connector: Reconnector, handler: Callable, logger: logging.Logger = None):
        assert isinstance(connector, Reconnector), 'connector must be Reconnector instance'
        self.logger = logger
        super().__init__(connector, handler)

    def run(self):
        while True:
            try:
                super(ReconConsumer, self).run()
                raise RuntimeError("Unreachable statement")
            except pika.exceptions.AMQPError:
                if self.logger:
                    self.logger.exception('RabbitMQ consumer exception')


class SafeConsumer(ReconConsumer):
    def term_sig_handler(self, signum, frame):
        self.received_signal = signum
        print(f'Received {signum} signal')
        if self.process_msg:
            self.got_term_signal = True
        else:
            print('Process was terminated')
            os._exit(signum)

    def add_signal_handlers(self):
        signal.signal(signal.SIGTERM, self.term_sig_handler)
        signal.signal(signal.SIGINT, self.term_sig_handler)

    def __init__(self, connector: Reconnector, handler: Callable, logger: logging.Logger = None):
        self.received_signal = None
        self.process_msg = False
        self.got_term_signal = False
        self.add_signal_handlers()

        def safe_wrapper(func):
            @functools.wraps(func)
            def wraps(*args, **kwargs):
                self.process_msg = True
                try:
                    result = func(*args, **kwargs)
                    if self.got_term_signal:
                        print('Process was terminated')
                        os._exit(self.received_signal or -1)
                    return result
                finally:
                    self.process_msg = False
            return wraps

        safe_handler = safe_wrapper(handler)
        super().__init__(connector, safe_handler, logger=logger)


class MultipleConsumer(Consumer):
    def __init__(self, connector: Connector, multi_handler: Callable):
        self.msg_limit = connector.prefetch_count
        self.mq_messages = []
        self.multi_handler = multi_handler
        super().__init__(connector, self.msg_handler)

    def msg_handler(self, channel, method, properties, body):
        self.mq_messages.append(MQMessage(channel, method, properties, body))
        if len(self.mq_messages) >= self.msg_limit or len(self.connector.channel._pending_events) == 0:
            self.multi_handler(self.mq_messages)
            while self.mq_messages:
                mq_msg = self.mq_messages.pop()
                mq_msg.channel.basic_ack(delivery_tag=mq_msg.method.delivery_tag)


class NotEmptyConsumer(Consumer):

    class CountCallback:
        def __init__(self, count: int, handler: Callable, finish_handler: Callable = None):
            self.count = count
            self.handler = handler
            self.finish_handler = finish_handler

        def __call__(self, ch, method, properties, body):
            self.handler(ch, method, properties, body)
            self.count -= 1
            if self.count <= 0:
                if self.finish_handler:
                    self.finish_handler()
                ch.stop_consuming()

    def __init__(self, connector: Connector, handler: Callable, finish_handler: Callable = None):
        self.finish_hadler = finish_handler
        super().__init__(connector, handler)

    def start_consuming(self):
        self.connector.create_connection()
        msg_count = self.connector.declared_queue.method.message_count
        if msg_count > 0:
            callback = self.CountCallback(msg_count, self.handler, finish_handler=self.finish_hadler)
            self.connector.channel.basic_consume(callback, queue=self.connector.queue)
            self.connector.channel.start_consuming()



from mq_consumer.connectors import Reconnector
from mq_consumer.consumers import NotEmptyConsumer

from tests.test_connection_params import connection_parameters


class TestConsumer(NotEmptyConsumer):
    def __init__(self):
        connector = Reconnector(
            connection_parameters,
            'exchange',
            'test',
        )
        super().__init__(connector, self.handle, finish_handler=lambda: print("FINISHED"))

    @staticmethod
    def handle(channel, method, properties, body):
        print(body)
        channel.basic_ack(delivery_tag=method.delivery_tag)


def run():
    TestConsumer().run()


if __name__ == '__main__':
    run()
